/* *****************************************************************************
  This file is part of the kde-game 'kjumpingCube'

  Copyright (C) 1998 by Matthias Kiefer <Matthias.Kiefer@stud.uni-karlsruhe.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  *******************************************************************************/  

#include <math.h>
#include <stdio.h>

#include "cubesim.h"
#include "cube.h"
#include "cubebox.h"
#include <qstring.h>

/******************************************************************************
*****			Class CubeSim   				  *****
******************************************************************************/

CubeSim::CubeSim(int owner,int value,int max)
{
   this->owner=owner;
   this->value=value;
   this->max=max;
}


void CubeSim::setOwner(int newOwner)
{
   owner=newOwner;
}

void CubeSim::setValue(int newValue)
{
   value=newValue;
}

void CubeSim::setMax(int newMax)
{
   max=newMax;
}

int CubeSim::getValue()
{
   return value;
}
 
int CubeSim::getOwner()
{
   return owner;
}  

int CubeSim::getMax()
{
   return max;
}


void CubeSim::decrease()
{
   value-=max;
}

void CubeSim::increase(int player)
{
   value++;
   owner=player;
}


bool CubeSim::overMax()
{
   if(value>max)
      return TRUE;
   return FALSE;
}

/******************************************************************************
*****			Class CubeSimBox 				  *****
******************************************************************************/


CubeSimBox::CubeSimBox(int newSize)
{
   int i,j;
   if(newSize<1)
   {
      debug("Error in initialisation of CubeSimBox");
      exit(1);   
   }
   
   size=newSize;
   
   cubes = new CubeSim*[size];
   for(i=0;i<size;i++)
      cubes[i]=new CubeSim[size];
 
   for(i=0;i<size;i++)
      for(j=0;j<size;j++)
      {
	 cubes[i][j].setMax(maxValue(i,j));
      }  
 
}


CubeSimBox::CubeSimBox(CubeSimBox *boxToCopy)
{
   int i,j;
   size=boxToCopy->size;

   cubes = new CubeSim*[size];
   for(i=0;i<size;i++)
      cubes[i]=new CubeSim[size];

   
   for(i=0;i<size;i++)
      for(j=0;j<size;j++)
	 cubes[i][j]=boxToCopy->cubes[i][j];
        
}

CubeSimBox::CubeSimBox(CubeSimBox &boxToCopy)
{
   int i,j;
   size=boxToCopy.size;

   cubes = new CubeSim*[size];
   for(i=0;i<size;i++)
      cubes[i]=new CubeSim[size];

   
   for(i=0;i<size;i++)
      for(j=0;j<size;j++)
	 cubes[i][j]=boxToCopy.cubes[i][j];
        
}
CubeSimBox::CubeSimBox(CubeBox *boxToCopy)
{
   int i,j;
   size=boxToCopy->dim();

   cubes = new CubeSim*[size];
   for(i=0;i<size;i++)
      cubes[i]=new CubeSim[size];

   
   for(i=0;i<size;i++)
      for(j=0;j<size;j++)
      {
         cubes[i][j].setValue(boxToCopy->cubes[i][j]->value());
	 cubes[i][j].setOwner(boxToCopy->cubes[i][j]->getOwner());
	 cubes[i][j].setMax(boxToCopy->cubes[i][j]->getMaxValue());
      }  
}

CubeSimBox::~CubeSimBox()
{
   int i;
   for(i=0;i<size;i++)
      delete [] cubes[i];
   
   delete [] cubes;
}


void CubeSimBox::copy(CubeBox *c)
{
   int i,j;
   
   if(size!=c->dim())
   {
      for(i=0;i<size;i++)
         delete [] cubes[i];
   
      delete [] cubes;
      
      size=c->dim();

      cubes = new CubeSim*[size];
      for(i=0;i<size;i++)
         cubes[i]=new CubeSim[size];
   }
   
   for(i=0;i<size;i++)
     for(j=0;j<size;j++)
     {
        cubes[i][j].setValue(c->cubes[i][j]->value());
        cubes[i][j].setOwner(c->cubes[i][j]->getOwner());
	cubes[i][j].setMax(c->cubes[i][j]->getMaxValue());
     }  
}



bool CubeSimBox::simulateMove(int row, int column,int player)
{
   bool finished;
   bool playerWon=0;
   
   if(cubes[row][column].getOwner()!=player && cubes[row][column].getOwner()!=0)
      return FALSE;
   
   cubes[row][column].increase(player);
   
   do
   {
      int i,j;
      finished=TRUE;
      playerWon=TRUE;
      
      // alle Cubes berprfen
      for(i=0;i<size;i++)
      {
	 for(j=0;j<size;j++)
         {
	    if(cubes[i][j].overMax())
	    {
	       increaseNeighbours(player,i,j);
	       cubes[i][j].decrease();
	       finished=FALSE;
	    }
	    
	    if(cubes[i][j].getOwner()!=player)
	       playerWon=FALSE;
         }  
      }  
      
      if(playerWon)
	return TRUE;	     
   }
   while(!finished);
   
   
   return TRUE;
}

// Funktion zum Bewerten des Feldes
int CubeSimBox::assessField(int player)
{
   int cubesOne=0;
   int cubesTwo=0;
   int pointsOne=0;
   int pointsTwo=0;
   int otherPlayer=(player==1)? 2:1;
   bool playerWon=TRUE; // flag, ob Spiel von player gewonnen
   bool otherPlayerWon=TRUE;
   
   int i,j;
   
   for(i=0;i<size;i++)
   {
      for(j=0;j<size;j++)
      {
	 if(cubes[i][j].getOwner()==1)
	 {
	    cubesOne++;
	    pointsOne+=(int)pow(cubes[i][j].getValue(),2);
	 }
	 else if(cubes[i][j].getOwner()==2)
	 {
	    cubesTwo++;
	    pointsTwo+=(int)pow(cubes[i][j].getValue(),2);
	 }
	 
	 if(cubes[i][j].getOwner()!=player)
	    playerWon=FALSE;
	 
	 if(cubes[i][j].getOwner()!=otherPlayer)
	    otherPlayerWon=FALSE;
      }  
     
   }
    
   
    
   if(player==1)
   {
      return (int)pow(cubesOne,2)+pointsOne-(int)pow(cubesTwo,2)-pointsTwo;
   }
   else
      return (int)pow(cubesTwo,2)+pointsTwo-(int)pow(cubesOne,2)-pointsOne;
}


bool CubeSimBox::playerWon(int player)
{
   int i,j;
   
   for(i=0;i<size;i++)
      for(j=0;j<size;j++)
      {
         if(cubes[i][j].getOwner()!=player)
	    return FALSE;
      }
      
   return TRUE;
}


CubeSim& CubeSimBox::cube(int row,int column)
{
   return cubes[row][column];
}



int CubeSimBox::maxValue(int row,int column)
{
   if(row == 0 || row == size-1)
   {
      if(column == 0 || column == size-1)  // falls Eckwrfel
	 return 2;
      else return 3;   // ansonsten Randwrfel
   }
   else if(column==0 || column == size-1)  // Randwrfel
      return 3;
   
   else 
      return 4;   // mittlere Wrfel
  
}


void CubeSimBox::increaseNeighbours(int player,int row, int column)
{
   if(row==0)
   {
    	if(column==0)  // linke obere Ecke
	{
	   cubes[0][1].increase(player);
	   cubes[1][0].increase(player);
	   return;
	}  
	else if(column==size-1)  // rechte obere Ecke
	{
	   cubes[0][size-2].increase(player);
	   cubes[1][size-1].increase(player);
	   return;
	}
	else  // oberer Rand
	{
	   cubes[0][column-1].increase(player);
	   cubes[0][column+1].increase(player);
	   cubes[1][column].increase(player);
	   return;
	}
   }
   else if(row==size-1) 
   {  
      if(column==0)  // linke untere Ecke
      {
         cubes[size-2][0].increase(player);
         cubes[size-1][1].increase(player);
         return;
      }
   
      else if(column==size-1) // rechte untere Ecke
      {
         cubes[size-2][size-1].increase(player);
         cubes[size-1][size-2].increase(player);
	 return;
      }
      else  // unterer Rand
      {
	cubes[size-1][column-1].increase(player);
	cubes[size-1][column+1].increase(player);
	cubes[size-2][column].increase(player);
	return; 
      }
   }
   else if(column==0) // linker Rand
   {
      cubes[row-1][0].increase(player);
      cubes[row+1][0].increase(player);
      cubes[row][1].increase(player);
      return;
   }
   else if(column==size-1)  // rechter Rand
   {
      cubes[row-1][size-1].increase(player);
      cubes[row+1][size-1].increase(player);
      cubes[row][size-2].increase(player);
      return;
   }
   else
   {
      cubes[row][column-1].increase(player);
      cubes[row][column+1].increase(player);
      cubes[row-1][column].increase(player);
      cubes[row+1][column].increase(player);
      return;
   }
   
      
}




